// This file is generated by kconfig_compiler_kf6 from radselectconfig.kcfg.
// All changes you do to this file will be lost.

#include "radselectconfig.h"

#include <qglobal.h>
#include <QFile>

class RadSelectConfigSkeletonHelper
{
  public:
    RadSelectConfigSkeletonHelper() noexcept : q(nullptr) {}
    ~RadSelectConfigSkeletonHelper() noexcept { delete q; q = nullptr; }
    RadSelectConfigSkeletonHelper(const RadSelectConfigSkeletonHelper&) = delete;
    RadSelectConfigSkeletonHelper& operator=(const RadSelectConfigSkeletonHelper&) = delete;
    RadSelectConfigSkeleton *q;
};
Q_GLOBAL_STATIC(RadSelectConfigSkeletonHelper, s_globalRadSelectConfigSkeleton)
RadSelectConfigSkeleton *RadSelectConfigSkeleton::self()
{
  if (!s_globalRadSelectConfigSkeleton()->q) {
    new RadSelectConfigSkeleton;
    s_globalRadSelectConfigSkeleton()->q->read();
  }

  return s_globalRadSelectConfigSkeleton()->q;
}

RadSelectConfigSkeleton::RadSelectConfigSkeleton( )
  : KConfigSkeleton( QStringLiteral( "kitenradselectrc" ) )
{
  Q_ASSERT(!s_globalRadSelectConfigSkeleton()->q);
  s_globalRadSelectConfigSkeleton()->q = this;
  setCurrentGroup( QStringLiteral( "Options" ) );


        QFont resultListFnt = QFontDatabase::systemFont(QFontDatabase::GeneralFont);
        resultListFnt.setPointSize( 25 );
      
  KConfigSkeleton::ItemFont  *itemResultListFont;
  itemResultListFont = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "resultListFont" ), mResultListFont, resultListFnt );
  addItem( itemResultListFont, QStringLiteral( "resultListFont" ) );
  KConfigSkeleton::ItemFont  *itemFont;
  itemFont = new KConfigSkeleton::ItemFont( currentGroup(), QStringLiteral( "font" ), mFont );
  addItem( itemFont, QStringLiteral( "font" ) );
  KConfigSkeleton::ItemBool  *itemSortByFrequency;
  itemSortByFrequency = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "sortByFrequency" ), mSortByFrequency, true );
  addItem( itemSortByFrequency, QStringLiteral( "sortByFrequency" ) );
  KConfigSkeleton::ItemBool  *itemCloseOnSearch;
  itemCloseOnSearch = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "closeOnSearch" ), mCloseOnSearch, false );
  addItem( itemCloseOnSearch, QStringLiteral( "closeOnSearch" ) );
}

RadSelectConfigSkeleton::~RadSelectConfigSkeleton()
{
  if (s_globalRadSelectConfigSkeleton.exists() && !s_globalRadSelectConfigSkeleton.isDestroyed()) {
    s_globalRadSelectConfigSkeleton()->q = nullptr;
  }
}

