/*
License: GPLv2
*/

/********************************************************************************
** Form generated from reading UI file 'radselectprefdialog.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RADSELECTPREFDIALOG_H
#define UI_RADSELECTPREFDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kfontrequester.h"

QT_BEGIN_NAMESPACE

class Ui_radselectprefdialog
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *kcfg_closeOnSearch;
    QCheckBox *kcfg_sortByFrequency;
    QGridLayout *gridLayout;
    QLabel *resultListFontSize_label;
    KFontRequester *kcfg_resultListFont;
    QLabel *fontSize_label;
    KFontRequester *kcfg_font;
    QSpacerItem *spacer;

    void setupUi(QWidget *radselectprefdialog)
    {
        if (radselectprefdialog->objectName().isEmpty())
            radselectprefdialog->setObjectName("radselectprefdialog");
        radselectprefdialog->resize(560, 466);
        vboxLayout = new QVBoxLayout(radselectprefdialog);
        vboxLayout->setContentsMargins(-1, -1, -1, -1);
        vboxLayout->setObjectName("vboxLayout");
        kcfg_closeOnSearch = new QCheckBox(radselectprefdialog);
        kcfg_closeOnSearch->setObjectName("kcfg_closeOnSearch");

        vboxLayout->addWidget(kcfg_closeOnSearch);

        kcfg_sortByFrequency = new QCheckBox(radselectprefdialog);
        kcfg_sortByFrequency->setObjectName("kcfg_sortByFrequency");

        vboxLayout->addWidget(kcfg_sortByFrequency);

        gridLayout = new QGridLayout();
        gridLayout->setContentsMargins(-1, -1, -1, -1);
        gridLayout->setObjectName("gridLayout");
        resultListFontSize_label = new QLabel(radselectprefdialog);
        resultListFontSize_label->setObjectName("resultListFontSize_label");

        gridLayout->addWidget(resultListFontSize_label, 0, 0, 1, 1);

        kcfg_resultListFont = new KFontRequester(radselectprefdialog);
        kcfg_resultListFont->setObjectName("kcfg_resultListFont");

        gridLayout->addWidget(kcfg_resultListFont, 0, 1, 1, 1);

        fontSize_label = new QLabel(radselectprefdialog);
        fontSize_label->setObjectName("fontSize_label");

        gridLayout->addWidget(fontSize_label, 1, 0, 1, 1);

        kcfg_font = new KFontRequester(radselectprefdialog);
        kcfg_font->setObjectName("kcfg_font");

        gridLayout->addWidget(kcfg_font, 1, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        spacer = new QSpacerItem(20, 20, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacer);


        retranslateUi(radselectprefdialog);

        QMetaObject::connectSlotsByName(radselectprefdialog);
    } // setupUi

    void retranslateUi(QWidget *radselectprefdialog)
    {
        radselectprefdialog->setWindowTitle(tr2i18n("Preferences", nullptr));
        kcfg_closeOnSearch->setText(tr2i18n("Close radselect when you press search", nullptr));
        kcfg_sortByFrequency->setText(tr2i18n("Display radicals in descending frequency order", nullptr));
        resultListFontSize_label->setText(tr2i18n("Search Results Font:", nullptr));
#if QT_CONFIG(tooltip)
        resultListFontSize_label->setProperty("toolTip", QVariant(tr2i18n("The font for the main kanji/kana entry in list view", nullptr)));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        resultListFontSize_label->setProperty("whatsThis", QVariant(tr2i18n("Use this to adjust the result list font", nullptr)));
#endif // QT_CONFIG(whatsthis)
        kcfg_resultListFont->setSampleText(tr2i18n("\346\274\242\345\255\227 \343\201\213\343\201\252", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_resultListFont->setProperty("toolTip", QVariant(tr2i18n("The font for the main kanji/kana entry in list view", nullptr)));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_resultListFont->setProperty("whatsThis", QVariant(tr2i18n("Use this to adjust the result list font", nullptr)));
#endif // QT_CONFIG(whatsthis)
        fontSize_label->setText(tr2i18n("Button Font:", nullptr));
#if QT_CONFIG(tooltip)
        fontSize_label->setProperty("toolTip", QVariant(tr2i18n("The font for the main kanji/kana entry in list view", nullptr)));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        fontSize_label->setProperty("whatsThis", QVariant(tr2i18n("Use this to adjust the button font", nullptr)));
#endif // QT_CONFIG(whatsthis)
        kcfg_font->setSampleText(tr2i18n("\346\274\242\345\255\227 \343\201\213\343\201\252", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_font->setProperty("toolTip", QVariant(tr2i18n("The font for the main kanji/kana entry in list view", nullptr)));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_font->setProperty("whatsThis", QVariant(tr2i18n("Use this to adjust the button font", nullptr)));
#endif // QT_CONFIG(whatsthis)
    } // retranslateUi

};

namespace Ui {
    class radselectprefdialog: public Ui_radselectprefdialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // RADSELECTPREFDIALOG_H

