/********************************************************************************
** Form generated from reading UI file 'kanjibrowserview.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KANJIBROWSERVIEW_H
#define UI_KANJIBROWSERVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcombobox.h"

QT_BEGIN_NAMESPACE

class Ui_KanjiBrowserView
{
public:
    QVBoxLayout *verticalLayout;
    QStackedWidget *_stackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout_2;
    KComboBox *_grades;
    KComboBox *_strokes;
    QListWidget *_kanjiList;
    QWidget *page_2;
    QGridLayout *gridLayout;
    QTextBrowser *_kanjiInformation;

    void setupUi(QWidget *KanjiBrowserView)
    {
        if (KanjiBrowserView->objectName().isEmpty())
            KanjiBrowserView->setObjectName("KanjiBrowserView");
        KanjiBrowserView->resize(700, 500);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KanjiBrowserView->sizePolicy().hasHeightForWidth());
        KanjiBrowserView->setSizePolicy(sizePolicy);
        KanjiBrowserView->setMinimumSize(QSize(0, 0));
        verticalLayout = new QVBoxLayout(KanjiBrowserView);
        verticalLayout->setObjectName("verticalLayout");
        _stackedWidget = new QStackedWidget(KanjiBrowserView);
        _stackedWidget->setObjectName("_stackedWidget");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(_stackedWidget->sizePolicy().hasHeightForWidth());
        _stackedWidget->setSizePolicy(sizePolicy1);
        _stackedWidget->setFrameShape(QFrame::NoFrame);
        _stackedWidget->setFrameShadow(QFrame::Plain);
        page = new QWidget();
        page->setObjectName("page");
        sizePolicy1.setHeightForWidth(page->sizePolicy().hasHeightForWidth());
        page->setSizePolicy(sizePolicy1);
        verticalLayout_2 = new QVBoxLayout(page);
        verticalLayout_2->setObjectName("verticalLayout_2");
        _grades = new KComboBox(page);
        _grades->setObjectName("_grades");

        verticalLayout_2->addWidget(_grades);

        _strokes = new KComboBox(page);
        _strokes->setObjectName("_strokes");

        verticalLayout_2->addWidget(_strokes);

        _kanjiList = new QListWidget(page);
        _kanjiList->setObjectName("_kanjiList");
        sizePolicy1.setHeightForWidth(_kanjiList->sizePolicy().hasHeightForWidth());
        _kanjiList->setSizePolicy(sizePolicy1);
        QFont font;
        font.setPointSize(24);
        _kanjiList->setFont(font);
        _kanjiList->setEditTriggers(QAbstractItemView::NoEditTriggers);
        _kanjiList->setProperty("showDropIndicator", QVariant(false));
        _kanjiList->setDragDropMode(QAbstractItemView::NoDragDrop);
        _kanjiList->setSelectionMode(QAbstractItemView::NoSelection);
        _kanjiList->setResizeMode(QListView::Adjust);
        _kanjiList->setViewMode(QListView::IconMode);

        verticalLayout_2->addWidget(_kanjiList);

        _stackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName("page_2");
        gridLayout = new QGridLayout(page_2);
        gridLayout->setObjectName("gridLayout");
        _kanjiInformation = new QTextBrowser(page_2);
        _kanjiInformation->setObjectName("_kanjiInformation");
        _kanjiInformation->setReadOnly(true);
        _kanjiInformation->setOverwriteMode(false);

        gridLayout->addWidget(_kanjiInformation, 0, 0, 1, 1);

        _stackedWidget->addWidget(page_2);

        verticalLayout->addWidget(_stackedWidget);


        retranslateUi(KanjiBrowserView);

        _stackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(KanjiBrowserView);
    } // setupUi

    void retranslateUi(QWidget *KanjiBrowserView)
    {
        KanjiBrowserView->setWindowTitle(tr2i18n("Kanji Browser", nullptr));
    } // retranslateUi

};

namespace Ui {
    class KanjiBrowserView: public Ui_KanjiBrowserView {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KANJIBROWSERVIEW_H

