/****************************************************************************
** Meta object code from reading C++ file 'kiten.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kiten-26.07.80/app/kiten.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kiten.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5KitenE_t {};
} // unnamed namespace

template <> constexpr inline auto Kiten::qt_create_metaobjectdata<qt_meta_tag_ZN5KitenE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kiten",
        "searchTextAndRaise",
        "",
        "str",
        "addExportListEntry",
        "index",
        "finishInit",
        "focusResultsView",
        "searchFromEdit",
        "searchText",
        "searchClipboard",
        "searchAndDisplay",
        "DictQuery",
        "searchInResults",
        "displayResults",
        "EntryList*",
        "radicalSearch",
        "kanjiBrowserSearch",
        "slotConfigure",
        "configureToolBars",
        "newToolBarConfig",
        "updateConfiguration",
        "loadDictionaries",
        "loadDictConfig",
        "print",
        "back",
        "forward",
        "goInHistory",
        "displayHistoryItem",
        "addHistory",
        "enableHistoryButtons",
        "setCurrentScrollValue",
        "value"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'searchTextAndRaise'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'addExportListEntry'
        QtMocHelpers::SlotData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'finishInit'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'focusResultsView'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'searchFromEdit'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'searchText'
        QtMocHelpers::SlotData<void(const QString &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'searchClipboard'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'searchAndDisplay'
        QtMocHelpers::SlotData<void(const DictQuery &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'searchInResults'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'displayResults'
        QtMocHelpers::SlotData<void(EntryList *)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 2 },
        }}),
        // Slot 'radicalSearch'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'kanjiBrowserSearch'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotConfigure'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'configureToolBars'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'newToolBarConfig'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateConfiguration'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadDictionaries'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'loadDictConfig'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'print'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'back'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'forward'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'goInHistory'
        QtMocHelpers::SlotData<void(int)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'displayHistoryItem'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'addHistory'
        QtMocHelpers::SlotData<void(EntryList *)>(29, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 2 },
        }}),
        // Slot 'enableHistoryButtons'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setCurrentScrollValue'
        QtMocHelpers::SlotData<void(int)>(31, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 32 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Kiten, qt_meta_tag_ZN5KitenE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kiten::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KitenE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KitenE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5KitenE_t>.metaTypes,
    nullptr
} };

void Kiten::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Kiten *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->searchTextAndRaise((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->addExportListEntry((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->finishInit(); break;
        case 3: _t->focusResultsView(); break;
        case 4: _t->searchFromEdit(); break;
        case 5: _t->searchText((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->searchClipboard(); break;
        case 7: _t->searchAndDisplay((*reinterpret_cast<std::add_pointer_t<DictQuery>>(_a[1]))); break;
        case 8: _t->searchInResults(); break;
        case 9: _t->displayResults((*reinterpret_cast<std::add_pointer_t<EntryList*>>(_a[1]))); break;
        case 10: _t->radicalSearch(); break;
        case 11: _t->kanjiBrowserSearch(); break;
        case 12: _t->slotConfigure(); break;
        case 13: _t->configureToolBars(); break;
        case 14: _t->newToolBarConfig(); break;
        case 15: _t->updateConfiguration(); break;
        case 16: _t->loadDictionaries(); break;
        case 17: _t->loadDictConfig((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->print(); break;
        case 19: _t->back(); break;
        case 20: _t->forward(); break;
        case 21: _t->goInHistory((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 22: _t->displayHistoryItem(); break;
        case 23: _t->addHistory((*reinterpret_cast<std::add_pointer_t<EntryList*>>(_a[1]))); break;
        case 24: _t->enableHistoryButtons(); break;
        case 25: _t->setCurrentScrollValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *Kiten::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kiten::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5KitenE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int Kiten::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 26)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 26;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 26)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 26;
    }
    return _id;
}
QT_WARNING_POP
