/********************************************************************************
** Form generated from reading UI file 'configfont.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGFONT_H
#define UI_CONFIGFONT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kfontrequester.h"

QT_BEGIN_NAMESPACE

class Ui_ConfigFont
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *Listview_mainentry_label;
    KFontRequester *kcfg_font;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *ConfigFont)
    {
        if (ConfigFont->objectName().isEmpty())
            ConfigFont->setObjectName("ConfigFont");
        ConfigFont->resize(546, 284);
        vboxLayout = new QVBoxLayout(ConfigFont);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        Listview_mainentry_label = new QLabel(ConfigFont);
        Listview_mainentry_label->setObjectName("Listview_mainentry_label");

        hboxLayout->addWidget(Listview_mainentry_label);

        kcfg_font = new KFontRequester(ConfigFont);
        kcfg_font->setObjectName("kcfg_font");

        hboxLayout->addWidget(kcfg_font);


        vboxLayout->addLayout(hboxLayout);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);


        retranslateUi(ConfigFont);

        QMetaObject::connectSlotsByName(ConfigFont);
    } // setupUi

    void retranslateUi(QWidget *ConfigFont)
    {
        ConfigFont->setWindowTitle(tr2i18n("Fonts", nullptr));
        Listview_mainentry_label->setText(tr2i18n("Main Font", nullptr));
        kcfg_font->setSampleText(tr2i18n("\346\274\242\345\255\227 \343\201\213\343\201\252 romaji", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ConfigFont: public Ui_ConfigFont {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGFONT_H

