/********************************************************************************
** Form generated from reading UI file 'configsearching.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGSEARCHING_H
#define UI_CONFIGSEARCHING_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_ConfigSearching
{
public:
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout;
    QCheckBox *kcfg_initialSearch;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *ConfigSearching)
    {
        if (ConfigSearching->objectName().isEmpty())
            ConfigSearching->setObjectName("ConfigSearching");
        ConfigSearching->resize(401, 348);
        hboxLayout = new QHBoxLayout(ConfigSearching);
        hboxLayout->setContentsMargins(-1, -1, -1, -1);
        hboxLayout->setObjectName("hboxLayout");
        vboxLayout = new QVBoxLayout();
        vboxLayout->setContentsMargins(-1, -1, -1, -1);
        vboxLayout->setObjectName("vboxLayout");
        kcfg_initialSearch = new QCheckBox(ConfigSearching);
        kcfg_initialSearch->setObjectName("kcfg_initialSearch");

        vboxLayout->addWidget(kcfg_initialSearch);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);


        hboxLayout->addLayout(vboxLayout);


        retranslateUi(ConfigSearching);

        QMetaObject::connectSlotsByName(ConfigSearching);
    } // setupUi

    void retranslateUi(QWidget *ConfigSearching)
    {
        ConfigSearching->setWindowTitle(tr2i18n("Searching", nullptr));
        kcfg_initialSearch->setText(tr2i18n("Search for \"jiten\" on startup", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ConfigSearching: public Ui_ConfigSearching {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGSEARCHING_H

