/********************************************************************************
** Form generated from reading UI file 'configdictselect.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGDICTSELECT_H
#define UI_CONFIGDICTSELECT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_configDictSelect
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QCheckBox *__useGlobal;
    QSpacerItem *spacer3_2;
    QHBoxLayout *hboxLayout1;
    QTreeWidget *fileList;
    QVBoxLayout *vboxLayout1;
    QPushButton *addButton;
    QPushButton *delButton;
    QSpacerItem *spacer4_2;

    void setupUi(QWidget *configDictSelect)
    {
        if (configDictSelect->objectName().isEmpty())
            configDictSelect->setObjectName("configDictSelect");
        configDictSelect->resize(458, 337);
        vboxLayout = new QVBoxLayout(configDictSelect);
        vboxLayout->setObjectName("vboxLayout");
        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout->setContentsMargins(0, 0, 0, 0);
#endif
        hboxLayout->setObjectName("hboxLayout");
        __useGlobal = new QCheckBox(configDictSelect);
        __useGlobal->setObjectName("__useGlobal");

        hboxLayout->addWidget(__useGlobal);

        spacer3_2 = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacer3_2);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1->setObjectName("hboxLayout1");
        fileList = new QTreeWidget(configDictSelect);
        fileList->setObjectName("fileList");

        hboxLayout1->addWidget(fileList);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        vboxLayout1->setObjectName("vboxLayout1");
        addButton = new QPushButton(configDictSelect);
        addButton->setObjectName("addButton");

        vboxLayout1->addWidget(addButton);

        delButton = new QPushButton(configDictSelect);
        delButton->setObjectName("delButton");

        vboxLayout1->addWidget(delButton);

        spacer4_2 = new QSpacerItem(20, 120, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout1->addItem(spacer4_2);


        hboxLayout1->addLayout(vboxLayout1);


        vboxLayout->addLayout(hboxLayout1);


        retranslateUi(configDictSelect);

        QMetaObject::connectSlotsByName(configDictSelect);
    } // setupUi

    void retranslateUi(QWidget *configDictSelect)
    {
        configDictSelect->setWindowTitle(tr2i18n("Dictionary Selector", nullptr));
        __useGlobal->setText(tr2i18n("Use prei&nstalled dict", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = fileList->headerItem();
        ___qtreewidgetitem->setText(1, tr2i18n("File", nullptr));
        ___qtreewidgetitem->setText(0, tr2i18n("Name", nullptr));
        addButton->setText(tr2i18n("&Add...", nullptr));
        delButton->setText(tr2i18n("Delete", nullptr));
    } // retranslateUi

};

namespace Ui {
    class configDictSelect: public Ui_configDictSelect {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGDICTSELECT_H

