/*
Used for the main window for kiten's radselect program.
*/

/********************************************************************************
** Form generated from reading UI file 'radical_selector.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RADICAL_SELECTOR_H
#define UI_RADICAL_SELECTOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_radical_selector
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *groupBox;
    QVBoxLayout *vboxLayout1;
    QListWidget *selected_radicals;
    QHBoxLayout *horizontalLayout;
    QLineEdit *copied_line;
    QPushButton *copy_button;
    QHBoxLayout *hboxLayout;
    QLabel *strokes_label_2;
    QSpinBox *strokes_low;
    QLabel *strokes_label;
    QSpinBox *strokes_high;
    QSpacerItem *spacerItem;
    QPushButton *clear_button;
    QScrollArea *radical_box;

    void setupUi(QWidget *radical_selector)
    {
        if (radical_selector->objectName().isEmpty())
            radical_selector->setObjectName("radical_selector");
        radical_selector->resize(608, 610);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(radical_selector->sizePolicy().hasHeightForWidth());
        radical_selector->setSizePolicy(sizePolicy);
        vboxLayout = new QVBoxLayout(radical_selector);
#ifndef UI_Q_OS_MACOS
        vboxLayout->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        vboxLayout->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout->setObjectName("vboxLayout");
        groupBox = new QGroupBox(radical_selector);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy1);
        vboxLayout1 = new QVBoxLayout(groupBox);
#ifndef UI_Q_OS_MACOS
        vboxLayout1->setSpacing(6);
#endif
#ifndef UI_Q_OS_MACOS
        vboxLayout1->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout1->setObjectName("vboxLayout1");
        selected_radicals = new QListWidget(groupBox);
        selected_radicals->setObjectName("selected_radicals");
        sizePolicy1.setHeightForWidth(selected_radicals->sizePolicy().hasHeightForWidth());
        selected_radicals->setSizePolicy(sizePolicy1);
        selected_radicals->setEditTriggers(QAbstractItemView::NoEditTriggers);
        selected_radicals->setDragDropMode(QAbstractItemView::NoDragDrop);
        selected_radicals->setMovement(QListView::Static);
        selected_radicals->setResizeMode(QListView::Adjust);
        selected_radicals->setViewMode(QListView::IconMode);

        vboxLayout1->addWidget(selected_radicals);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        copied_line = new QLineEdit(groupBox);
        copied_line->setObjectName("copied_line");

        horizontalLayout->addWidget(copied_line);

        copy_button = new QPushButton(groupBox);
        copy_button->setObjectName("copy_button");

        horizontalLayout->addWidget(copy_button);


        vboxLayout1->addLayout(horizontalLayout);


        vboxLayout->addWidget(groupBox);

        hboxLayout = new QHBoxLayout();
#ifndef UI_Q_OS_MACOS
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        strokes_label_2 = new QLabel(radical_selector);
        strokes_label_2->setObjectName("strokes_label_2");
        strokes_label_2->setAlignment(Qt::AlignCenter);

        hboxLayout->addWidget(strokes_label_2);

        strokes_low = new QSpinBox(radical_selector);
        strokes_low->setObjectName("strokes_low");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(strokes_low->sizePolicy().hasHeightForWidth());
        strokes_low->setSizePolicy(sizePolicy2);

        hboxLayout->addWidget(strokes_low);

        strokes_label = new QLabel(radical_selector);
        strokes_label->setObjectName("strokes_label");
        strokes_label->setAlignment(Qt::AlignCenter);

        hboxLayout->addWidget(strokes_label);

        strokes_high = new QSpinBox(radical_selector);
        strokes_high->setObjectName("strokes_high");
        sizePolicy2.setHeightForWidth(strokes_high->sizePolicy().hasHeightForWidth());
        strokes_high->setSizePolicy(sizePolicy2);
        strokes_high->setWrapping(true);

        hboxLayout->addWidget(strokes_high);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        clear_button = new QPushButton(radical_selector);
        clear_button->setObjectName("clear_button");
        sizePolicy2.setHeightForWidth(clear_button->sizePolicy().hasHeightForWidth());
        clear_button->setSizePolicy(sizePolicy2);

        hboxLayout->addWidget(clear_button);


        vboxLayout->addLayout(hboxLayout);

        radical_box = new QScrollArea(radical_selector);
        radical_box->setObjectName("radical_box");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Expanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(radical_box->sizePolicy().hasHeightForWidth());
        radical_box->setSizePolicy(sizePolicy3);
        radical_box->setFrameShape(QFrame::StyledPanel);
        radical_box->setFrameShadow(QFrame::Raised);

        vboxLayout->addWidget(radical_box);

        QWidget::setTabOrder(clear_button, selected_radicals);

        retranslateUi(radical_selector);

        QMetaObject::connectSlotsByName(radical_selector);
    } // setupUi

    void retranslateUi(QWidget *radical_selector)
    {
        radical_selector->setWindowTitle(tr2i18n("RadicalSelector", nullptr));
        groupBox->setTitle(tr2i18n("Results", nullptr));
        copy_button->setText(tr2i18n("&To Clipboard", nullptr));
        strokes_label_2->setText(tr2i18n("Strokes:", nullptr));
        strokes_label->setText(tr2i18n(" - ", nullptr));
        clear_button->setText(tr2i18n("Clear", nullptr));
    } // retranslateUi

};

namespace Ui {
    class radical_selector: public Ui_radical_selector {};
} // namespace Ui

QT_END_NAMESPACE

#endif // RADICAL_SELECTOR_H

