/*
License is GPL
*/

/********************************************************************************
** Form generated from reading UI file 'configsorting.ui'
**
** Created by: Qt User Interface Compiler version 6.12.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONFIGSORTING_H
#define UI_CONFIGSORTING_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kactionselector.h"

QT_BEGIN_NAMESPACE

class Ui_configSorting
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *kcfg_dictionary_enable;
    KActionSelector *dictionary_order;
    QFrame *separator;
    KActionSelector *field_order;

    void setupUi(QWidget *configSorting)
    {
        if (configSorting->objectName().isEmpty())
            configSorting->setObjectName("configSorting");
        configSorting->resize(497, 381);
        vboxLayout = new QVBoxLayout(configSorting);
        vboxLayout->setObjectName("vboxLayout");
        kcfg_dictionary_enable = new QCheckBox(configSorting);
        kcfg_dictionary_enable->setObjectName("kcfg_dictionary_enable");

        vboxLayout->addWidget(kcfg_dictionary_enable);

        dictionary_order = new KActionSelector(configSorting);
        dictionary_order->setObjectName("dictionary_order");

        vboxLayout->addWidget(dictionary_order);

        separator = new QFrame(configSorting);
        separator->setObjectName("separator");
        separator->setFrameShape(QFrame::HLine);
        separator->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(separator);

        field_order = new KActionSelector(configSorting);
        field_order->setObjectName("field_order");

        vboxLayout->addWidget(field_order);


        retranslateUi(configSorting);

        QMetaObject::connectSlotsByName(configSorting);
    } // setupUi

    void retranslateUi(QWidget *configSorting)
    {
        configSorting->setWindowTitle(tr2i18n("sorting", nullptr));
        kcfg_dictionary_enable->setText(tr2i18n("Sort Se&arch Results By Dictionary", nullptr));
        dictionary_order->setAvailableLabel(tr2i18n("Dicti&onaries Available:", nullptr));
        dictionary_order->setSelectedLabel(tr2i18n("Displa&y First:", nullptr));
        field_order->setAvailableLabel(tr2i18n("Fields to Sort &By:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class configSorting: public Ui_configSorting {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONFIGSORTING_H

