// This file is generated by kconfig_compiler_kf6 from kanjibrowserconfig.kcfg.
// All changes you do to this file will be lost.
#ifndef KANJIBROWSERCONFIGSKELETON_H
#define KANJIBROWSERCONFIGSKELETON_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

#include <kfontrequester.h>
#include <QFontDatabase>
#include <qfont.h>

class KanjiBrowserConfigSkeleton : public KConfigSkeleton
{
  public:

    static KanjiBrowserConfigSkeleton *self();
    ~KanjiBrowserConfigSkeleton() override;

    /**
      Set kanjiListFont
    */
    static
    void setKanjiListFont( const QFont & v )
    {
      if (!self()->isKanjiListFontImmutable())
        self()->mKanjiListFont = v;
    }

    /**
      Get kanjiListFont
    */
    static
    QFont kanjiListFont()
    {
      return self()->mKanjiListFont;
    }

    /**
      Is kanjiListFont Immutable
    */
    static
    bool isKanjiListFontImmutable()
    {
      return self()->isImmutable( QStringLiteral( "kanjiListFont" ) );
    }

    /**
      Set kanjiSize
    */
    static
    void setKanjiSize( double v )
    {
      if (!self()->isKanjiSizeImmutable())
        self()->mKanjiSize = v;
    }

    /**
      Get kanjiSize
    */
    static
    double kanjiSize()
    {
      return self()->mKanjiSize;
    }

    /**
      Is kanjiSize Immutable
    */
    static
    bool isKanjiSizeImmutable()
    {
      return self()->isImmutable( QStringLiteral( "kanjiSize" ) );
    }

    /**
      Set kanaFont
    */
    static
    void setKanaFont( const QFont & v )
    {
      if (!self()->isKanaFontImmutable())
        self()->mKanaFont = v;
    }

    /**
      Get kanaFont
    */
    static
    QFont kanaFont()
    {
      return self()->mKanaFont;
    }

    /**
      Is kanaFont Immutable
    */
    static
    bool isKanaFontImmutable()
    {
      return self()->isImmutable( QStringLiteral( "kanaFont" ) );
    }

    /**
      Set labelFont
    */
    static
    void setLabelFont( const QFont & v )
    {
      if (!self()->isLabelFontImmutable())
        self()->mLabelFont = v;
    }

    /**
      Get labelFont
    */
    static
    QFont labelFont()
    {
      return self()->mLabelFont;
    }

    /**
      Is labelFont Immutable
    */
    static
    bool isLabelFontImmutable()
    {
      return self()->isImmutable( QStringLiteral( "labelFont" ) );
    }

  protected:
    KanjiBrowserConfigSkeleton();
    friend class KanjiBrowserConfigSkeletonHelper;


    // Options
    QFont mKanjiListFont;
    double mKanjiSize;
    QFont mKanaFont;
    QFont mLabelFont;
};

#endif

