// This file is generated by kconfig_compiler_kf6 from radselectconfig.kcfg.
// All changes you do to this file will be lost.
#ifndef RADSELECTCONFIGSKELETON_H
#define RADSELECTCONFIGSKELETON_H

#include <kconfigskeleton.h>
#include <QCoreApplication>
#include <QDebug>

#include <kfontrequester.h>
#include <QFontDatabase>
#include <qfont.h>

class RadSelectConfigSkeleton : public KConfigSkeleton
{
  public:

    static RadSelectConfigSkeleton *self();
    ~RadSelectConfigSkeleton() override;

    /**
      Set resultListFont
    */
    static
    void setResultListFont( const QFont & v )
    {
      if (!self()->isResultListFontImmutable())
        self()->mResultListFont = v;
    }

    /**
      Get resultListFont
    */
    static
    QFont resultListFont()
    {
      return self()->mResultListFont;
    }

    /**
      Is resultListFont Immutable
    */
    static
    bool isResultListFontImmutable()
    {
      return self()->isImmutable( QStringLiteral( "resultListFont" ) );
    }

    /**
      Set font
    */
    static
    void setFont( const QFont & v )
    {
      if (!self()->isFontImmutable())
        self()->mFont = v;
    }

    /**
      Get font
    */
    static
    QFont font()
    {
      return self()->mFont;
    }

    /**
      Is font Immutable
    */
    static
    bool isFontImmutable()
    {
      return self()->isImmutable( QStringLiteral( "font" ) );
    }

    /**
      Set Display radicals in descending frequency order
    */
    static
    void setSortByFrequency( bool v )
    {
      if (!self()->isSortByFrequencyImmutable())
        self()->mSortByFrequency = v;
    }

    /**
      Get Display radicals in descending frequency order
    */
    static
    bool sortByFrequency()
    {
      return self()->mSortByFrequency;
    }

    /**
      Is Display radicals in descending frequency order Immutable
    */
    static
    bool isSortByFrequencyImmutable()
    {
      return self()->isImmutable( QStringLiteral( "sortByFrequency" ) );
    }

    /**
      Set Do we close this program when searching?
    */
    static
    void setCloseOnSearch( bool v )
    {
      if (!self()->isCloseOnSearchImmutable())
        self()->mCloseOnSearch = v;
    }

    /**
      Get Do we close this program when searching?
    */
    static
    bool closeOnSearch()
    {
      return self()->mCloseOnSearch;
    }

    /**
      Is Do we close this program when searching? Immutable
    */
    static
    bool isCloseOnSearchImmutable()
    {
      return self()->isImmutable( QStringLiteral( "closeOnSearch" ) );
    }

  protected:
    RadSelectConfigSkeleton();
    friend class RadSelectConfigSkeletonHelper;


    // Options
    QFont mResultListFont;
    QFont mFont;
    bool mSortByFrequency;
    bool mCloseOnSearch;
};

#endif

